import java.awt.event.*;
import java.awt.*;

public class TrackSOS extends Canvas implements Runnable {

/*
*********************************************************
CLASS VARIABLES
*********************************************************
*/
    Image position, sos;
    Graphics positiong, sosg;
    double dtime=0.001, time=0.0;
    public int scale,sxscale,syscale;
    Triple pos = new Triple();
    Triple oldpos = new Triple();
    Triple vel = new Triple();
    Triple oldvel = new Triple();
    Triple acc = new Triple();
    Triple oldacc = new Triple();
    double accmag,dacc;
    int sleeptime=5;
    double ycross,ycy,ycvy;
    double galmass=1.0;
    double galshape=1.0;
    double galcore=0.25;
    double orbenergy;
    double rmax,vesc0;
    double y,vy,yold,vyold;
    int galtype=Galaxy.SINGULAR;
    boolean blackHole=true;
    boolean running; 
    boolean doReset=true;

    Galaxy gal;
/*
*********************************************************
CONSTRUCTOR
*********************************************************
*/

    TrackSOS() {
          this(Galaxy.SINGULAR,1.0,-1.0,false,0.1,1.2);
    }
   

    TrackSOS(int GType, double GShape, double GEnergy, boolean GBH){
              this(GType, GShape, GEnergy,GBH,0.1,1.2);
    }
    
    TrackSOS(int GType, double GShape, double GEnergy, boolean GBH, 
             double y, double vy){
        setSim(GType, GShape, GEnergy, GBH, y, vy);
    }

    public void setSim(int GType, double GShape, double GEnergy, 
                           boolean GBH, double y, double vy)  {
        galtype=GType;galshape=GShape;orbenergy=GEnergy;blackHole=GBH;
    
	gal = new Galaxy(galmass, galshape, galcore, galtype, blackHole);

	rmax=gal.maxRadius(orbenergy);
	vesc0=gal.maxVel(0.0, 0.05, orbenergy);

	scale=(int) (130/rmax);
	sxscale=(int) (130/rmax/galshape);
	syscale=(int) (125/vesc0);
	
	pos=new Triple(0,y/sxscale,0);
	vel=new Triple(0,vy/syscale,0);
	oldacc=gal.Acceleration(pos);
	vel.x=gal.initVx(pos.y,vel.y,orbenergy);
	vel.z=0.0;
    
    }
    
    Color sosColor;
    public void setSOSColor(Color c) {sosColor=c;}
    
    Canvas mySOS;
    public void setSOS(Canvas c) {mySOS=c;}//Pointers...
   
    public void stop(){
	running = false;
    }
   
    Thread foo;

	public void resetSOS(){


	    sos = this.createImage(mySOS.getSize().width,mySOS.getSize().height);
	    sosg = sos.getGraphics();
	    sosg.translate(sos.getWidth(null)/2, sos.getHeight(null)/2);        
	    sosg.setColor(Color.white);
	    sosg.drawLine(0,-200,0,200);
	    sosg.drawLine(-200,0,200,0);
	    sosg.drawString("Y",130,12);
	    sosg.drawString("VY",3,135);

	    yold=0.05;
	    vyold=vesc0;
	    for (int i=1; i<100; i++) {
		y=i*(rmax/100);
		vy=Math.sqrt(2.0*(orbenergy-gal.Potential(0.,y)));
		sosg.drawLine( (int)(yold*sxscale), (int)(vyold*syscale), 
                               (int)(y*sxscale),(int)(vy*syscale));
		sosg.drawLine( (int)(-yold*sxscale), (int)(vyold*syscale), 
                               (int)(-y*sxscale),(int)(vy*syscale));
		sosg.drawLine( (int)(yold*sxscale), (int)(-vyold*syscale), 
                               (int)(y*sxscale),(int)(-vy*syscale));
		sosg.drawLine( (int)(-yold*sxscale), (int)(-vyold*syscale),
                               (int)(-y*sxscale),(int)(-vy*syscale));
		yold=y;
		vyold=vy;
	    }
	    mySOS.getGraphics().drawImage(sos,0,0,null);
	}

    public void go() {

	int i;

	if (!running) {
	    position = this.createImage(this.getSize().width,this.getSize().height);
	    positiong = position.getGraphics();
	    positiong.clearRect(0, 0, this.getSize().width, this.getSize().height);
	    positiong.translate(position.getWidth(null)/2, position.getHeight(null)/2);	  
	    positiong.setColor(Color.white);
	    positiong.drawLine(0,-200,0,200);
	    positiong.drawLine(-200,0,200,0);
	    positiong.drawString("X",130,12);
	    positiong.drawString("Y",3,135);
	    positiong.setColor(Color.red);
	    
	  if(doReset){
		resetSOS();
	    doReset=false;
      }
      
      sosg.setColor(sosColor);
      
	    foo = new Thread(this);
	    running = true;
	    foo.start();
	}
    }
    
    private class painter extends Thread{
        private TrackSOS parent;
        public boolean running=true;
        painter(TrackSOS p){parent=p;}
        public void run(){
           while (running) {
            try{
                sleep(10);
            }
            catch (Exception e){System.out.println("painter.run: " + e.getClass().getName() + " " + e.getMessage());}
            parent.repaint();
           }
        }
    }
/*
*********************************************************
RUN THREAD
*********************************************************
*/
    public void run() {
    painter paintfoo = new painter(this);
    paintfoo.start();
	while(running) {

	    pos.copy(oldpos);
	    vel.copy(oldvel);
	    acc.copy(oldacc);

	    acc=gal.Acceleration(pos);
	    accmag=Triple.TripleMag(acc);
	    dacc=Triple.TripleMag(Triple.SubTriple(acc,oldacc));
	    
	    gal.RK4(pos,vel,dtime);

	    time += dtime;

	    dtime = 1.e-5*(accmag/dacc);
            dtime = Math.min(dtime,0.01);
            dtime = Math.max(dtime,1.e-5);

	    positiong.drawLine(
			       (int)(oldpos.x*scale),
			       (int)(oldpos.y*scale),
			       (int)(pos.x*scale),
			       (int)(pos.y*scale));

	    ycross=pos.x*oldpos.x;

	    if (ycross<=0) {
		ycy=0.5*(pos.y+oldpos.y);
		ycvy=0.5*(vel.y+oldvel.y);
		sosg.fillOval((int)(ycy*sxscale), (int)(ycvy*syscale), 5, 5);
	    }

	    try{ 
		foo.sleep(sleeptime);
	    }
	    catch (InterruptedException e) {System.out.println("tracksos.run: " + e.getClass().getName() + " " + e.getMessage());} /**/

	}//while
    paintfoo.running=false;
    } //run

    public void update(Graphics g){
        paint(g);
    }
    
    public void paint(Graphics g) {
	if ((g != null) & (position != null)) {
            g.drawImage(position, 0, 0, null);
            g.setColor(Color.blue);
            g.fillOval((int)(pos.x*scale+this.getBounds().width/2),(int)(pos.y*scale+this.getBounds().height/2), 5, 5);
	}

	mySOS.getGraphics().drawImage(sos,0,0,null);

    }
    public boolean isRunning(){return(running);}
    public void setSleep(int s){sleeptime=s;}
    public void setPosVel(Triple p, Triple v){pos=p;vel=v;}
}




















